//
//  DetailsTableViewController.swift
//  Restaurants
//
//  Created by Radu Ionescu on 22/03/2017.
//  Copyright © 2017 lab. All rights reserved.
//

import UIKit
import CoreLocation

class DetailsTableViewController: UITableViewController
{
    var appModel: AppModel?
    var restaurant:[String:Any]?

    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var nameLabel: UILabel!
    @IBOutlet weak var addressLabel: UILabel!
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        self.appModel = AppModel.sharedModel
        self.nameLabel!.text = (self.restaurant?["name"] as! String)

        self.reloadImage()
        self.reloadAddress()
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func reloadImage()
    {
        let imgURLString = self.restaurant?["imgURL"] as! String
        
        if self.appModel!.images![imgURLString] != nil
        {
            let imgData = self.appModel!.images![imgURLString]
            self.imageView?.image = UIImage(data: imgData!)
        }
        else {
            
            let queue = DispatchQueue(label: "Image Downloader")
            queue.async { [unowned self] in
                
                let imgURL = URL(string: imgURLString)
                
                do {
                    
                    let imgData = try Data(contentsOf: imgURL!)
                    self.appModel!.images![imgURLString] = imgData
                    
                    DispatchQueue.main.async {
                        
                        self.imageView?.image = UIImage(data: imgData)
                    }
                }
                catch
                {
                    
                }
            }
        }
    }
    
    func reloadAddress()
    {
        let geocoder = CLGeocoder()
        let location = CLLocation(latitude: restaurant!["latitude"] as! CLLocationDegrees, longitude: restaurant!["longitude"] as! CLLocationDegrees)
        geocoder.reverseGeocodeLocation(location) { [unowned self] (placemarks, error) in
            
            if error == nil
            {
                let placemark = placemarks?.first
                
                if let street = placemark?.thoroughfare, let city = placemark?.locality
                {
                    self.addressLabel!.text = "\(street), \(city)"
                }
                else
                {
                    self.addressLabel!.text = "Unknown Address"
                }
            }
            else
            {
                self.addressLabel!.text = "Unknown Address"
            }
        }
    }
}
