//
//  ViewController.swift
//  Restaurants
//
//  Created by Radu Ionescu on 22/03/2017.
//  Copyright © 2017 lab. All rights reserved.
//

import UIKit

class ViewController: UIViewController, UITableViewDelegate, UITableViewDataSource
{
    var appModel: AppModel?

    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        self.appModel = AppModel.sharedModel
        
        self.activityIndicator!.startAnimating()
        self.reloadData()
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(ViewController.reloadData),
                                               name: Notification.Name(rawValue: "didReceiveRestaurantsData"),
                                               object: appModel!)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func reloadData()
    {
        self.tableView!.reloadData()
        
        if (appModel?.restaurants?.count ?? 0) > 0
        {
            self.activityIndicator!.stopAnimating()
        }
    }
    
    // MARK: - Table View Data Source Methods
    
    func numberOfSections(in tableView: UITableView) -> Int
    {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        return appModel?.restaurants?.count ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell = tableView.dequeueReusableCell(withIdentifier: "RestaurantCell", for: indexPath) as! RestaurantTableViewCell
        
        let restaurant = appModel?.restaurants?[indexPath.row] as! [String: Any]
        
        cell.nameLabel?.text = (restaurant["name"] as! String)
        cell.categoryLabel?.text = (restaurant["category"] as! String)
        
        
        cell.tag = indexPath.row
        let imgURLString = restaurant["imgURL"] as! String
        
        if appModel!.images![imgURLString] != nil
        {
            let imgData = appModel!.images![imgURLString]
            cell.thumbImageView?.image = UIImage(data: imgData!)
        }
        else
        {
            let queue = DispatchQueue(label: "Image Downloader")
            queue.async { [unowned self] in
                
                let imgURL = URL(string: imgURLString)
                
                do {
                    let imgData = try Data(contentsOf: imgURL!)
                    self.appModel!.images![imgURLString] = imgData
    
                    DispatchQueue.main.async {
                        
                        if cell.tag == indexPath.row
                        {
                            cell.thumbImageView?.image = UIImage(data: imgData)
                        }
                    }
                }
                catch
                {
                    
                }
            }
        }
        
        return cell
    }
    
    // MARK: - Table View Delegate Methods
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath)
    {
        tableView.deselectRow(at: indexPath, animated: true)
    }
    
    // MARK: - Segues
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        let indexPath = tableView.indexPath(for: sender as! UITableViewCell)
        let restaurant = appModel?.restaurants?[(indexPath?.row)!] as! [String: Any]
        
        let detailsViewController = segue.destination as! DetailsTableViewController
        detailsViewController.restaurant = restaurant
    }
}
